machines

package
v0.0.0-...-591ba7b Latest Latest
Warning

This package is not in the latest version of its module.

Go to latest
Published: Apr 17, 2026 License: Apache-2.0 Imports: 23 Imported by: 0

Documentation

Index

Constants

This section is empty.

Variables

This section is empty.

Functions

func NewPipelineWebhook

func NewPipelineWebhook() lib.PipelineAdmissionWebhook

Create a new pipeline admission webhook for the machines scheduling domain, using the known filters, weighers and detectors for validation.

Types

type FilterWeigherPipelineController

type FilterWeigherPipelineController struct {
	// Toolbox shared between all pipeline controllers.
	lib.BasePipelineController[lib.FilterWeigherPipeline[ironcore.MachinePipelineRequest]]

	// Monitor to pass down to all pipelines.
	Monitor lib.FilterWeigherPipelineMonitor
	// contains filtered or unexported fields
}

The decision pipeline controller takes decision resources containing a machine ref and runs the scheduling pipeline to make a decision. This decision is then written back to the decision resource status.

Additionally, the controller watches for pipeline and step changes to reconfigure the pipelines as needed.

func (*FilterWeigherPipelineController) InitPipeline

The base controller will delegate the pipeline creation down to this method.

func (*FilterWeigherPipelineController) PipelineType

The type of pipeline this controller manages.

func (*FilterWeigherPipelineController) ProcessNewMachine

func (c *FilterWeigherPipelineController) ProcessNewMachine(ctx context.Context, machine *ironcorev1alpha1.Machine) error

func (*FilterWeigherPipelineController) Reconcile

func (*FilterWeigherPipelineController) SetupWithManager

Directories

Path Synopsis
plugins

Jump to